# -*- coding: utf-8 -*-
import logging

from django.contrib.auth.backends import ModelBackend
from django.db import transaction

from sentry.models import User, UserEmail, Organization, OrganizationMember
from sentry.models import OrganizationMemberTeam, Team
from sentry.models import AuditLogEntry, AuditLogEntryEvent

import crowd

from .constants import CROWD_URL, CROWD_APP_NAME, CROWD_APP_PASSWORD

def new_user(username, success):
    # see https://github.com/getsentry/sentry/blob/master/src/sentry/auth/helper.py
    name = success['name']
    email = success['email']
    with transaction.atomic():
        user = User.objects.create(
            username=username,
            email=email,
            name=name,
            is_managed=True,
        )

        organization = Organization.get_default()
        om = OrganizationMember.objects.create(
            organization=organization,
            role=organization.default_role,
            user=user,
            flags=OrganizationMember.flags['sso:linked'],
        )

        AuditLogEntry.objects.create(
            organization=organization,
            actor=user,
            # TODO attribute request ip
            #ip=
            target_object=om.id,
            target_user=om.user,
            event=AuditLogEntryEvent.MEMBER_ADD,
            data=om.get_audit_log_data(),
        )
        return user


class SentryCrowdBackend(ModelBackend):
    """
    Mimick email backend and authenticate against Crowd server.

    Auto create account if not exists already.
    """
    # According to https://github.com/getsentry/sentry/blob/21.9.0/src/sentry/conf/server.py
    # Only email backend is supported by "default"
    # See https://github.com/getsentry/sentry/blob/21.9.0/src/sentry/utils/auth.py 
    # for structure of Email backend.
    def __init__(self, *args, **kwargs):
        self._crowd = crowd.CrowdServer(CROWD_URL, CROWD_APP_NAME, CROWD_APP_PASSWORD)
        super(SentryCrowdBackend, self).__init__(*args, **kwargs)

    def authenticate(self, request, username=None, password=None):
        success = self._crowd.auth_user(username, password)
        if success:
            try:
                user = User.objects.filter(is_active=True).exclude(password='!').get(username=username)
            except User.DoesNotExist:
                user = new_user(username, success)
            return user
        return None

    def user_can_authenticate(self, user):
        return True


# -*- coding: utf-8 -*-
from django.conf import settings

CROWD_URL = getattr(settings, 'CROWD_URL', None)
CROWD_APP_NAME = getattr(settings, 'CROWD_APP_NAME', None)
CROWD_APP_PASSWORD = getattr(settings, 'CROWD_APP_PASSWORD', None)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
sentry-auth-crowd
==================
Original work by 2016 Bas van Oostveen, updated for Sentry 21.9.0 by NWK
:copyright: (c) 2016 Bas van Oostveen, 2021 NWK
"""

from setuptools import setup, find_packages


install_requires = [
    'sentry>=21.9.0',
    'Crowd>=2.0.1',
]

tests_require = [
    'flake8>=2.0,<2.1',
]

setup(
    name='sentry-auth-crowd-NWK',
    version='0.7.0',
    author='NWK',
    author_email='NWKDenmark@protonmail.com',
    url='https://github.com/NWKDenmark/sentry-auth-crowd',
    description='Crowd authentication provider for Sentry',
    long_description=__doc__,
    license='Apache 2.0',
    packages=find_packages(exclude=['tests']),
    zip_safe=False,
    install_requires=install_requires,
    tests_require=tests_require,
    extras_require={'tests': tests_require},
    include_package_data=True,

    classifiers=[
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Operating System :: OS Independent',
        'Topic :: Software Development'
    ],
)
